"""
Tools for sending e-mail.
"""
from OpenSSL import SSL

from twisted.internet import defer, ssl
from twisted.mail import smtp


_CONTEXT_FACTORY = ssl.ClientContextFactory()
_CONTEXT_FACTORY.method = SSL.TLSv1_METHOD



class Mailer(object):
    """
    Sends e-mail.
    """
    def __init__(self, endpoint, credentials):
        self._endpoint = endpoint
        self._authentication = [credentials.username, credentials.password]
        self._factoryKwargs = {"contextFactory": _CONTEXT_FACTORY}


    def send(self, sender, recipient, content):
        d = defer.Deferred()

        args = self._authentication + [sender, recipient, content, d]
        factory = smtp.ESMTPSenderFactory(*args, **self._factoryKwargs)

        return self.endpoint.connect(factory).addCallback(lambda _ign: d)



from email import Charset, generator, header
from email.mime import multipart, text

try:
    from cStringIO import StringIO
except ImportError:
    from StringIO import StringIO


# Yay global state!
Charset.add_charset('utf-8', Charset.QP, Charset.QP, 'utf-8')


def buildMessage(headers, parts):
    message = multipart.MIMEMultipart('alternative')

    for name, value in headers.iteritems():
        multipart[name] = _encodeHeader(value)

    for partType, part in parts.iteritems():
        mt = text.MIMEText(part.encode("utf-8"), partType, "UTF-8").encode()
        message.attach(mt)

    return message


def _encodeHeader(headerValue):
    """
    Encodes a header value.

    Returns ASCII if possible, else returns an UTF-8 encoded e-mail header.
    """
    try:
        return headerValue.encode('ascii', 'strict')
    except UnicodeError:
        encoded = headerValue.encode("utf-8")
        return header.Header(encoded, "UTF-8").encode()


def messageToFile(message):
    """
    Flattens a message into a file-like object.
    """
    outFile = StringIO()
    messageGenerator = generator.Generator(outFile, False)
    messageGenerator.flatten(message)
    outFile.seek(0, 0)
    return outFile



"""
Interface definition.
"""
from zope import interface


class IMailer(interface.Interface):
    def send(sender, recipient, content):
        """
        Sends the content over e-mail to the recipient as the sender.
        """



import lxml.html
import lxml.html.clean


def textFromHTML(html):
    """
    Cleans and parses text from the given HTML.
    """
    cleaner = lxml.html.clean.Cleaner(scripts=True)
    cleaned = cleaner.clean_html(html)
    return lxml.html.fromstring(cleaned).text_content()



#!/usr/bin/env python
from setuptools import find_packages, setup

setup(name='txeasymail',
      version='20120927',
      description='Twisted client for easy email sending',
      url='https://github.com/lvh/txeasymail',

      author='Laurens Van Houtven',
      author_email='_@lvh.cc',

      packages=find_packages(),

      install_requires=['twisted'],

      license='ISC',
      classifiers=[
        "Development Status :: 3 - Alpha",
        "Framework :: Twisted",
        "License :: OSI Approved :: ISC License (ISCL)",
        ])
